/*
 * Copyright 2013 University of Chicago and Argonne National Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License
 */

/*
  BLOB.H

  Binary Large OBjects

  The Tcl bindings for all this are generated by SWIG
*/

/**
   Simple struct for variable-length data blob

   Based on this struct, SWIG will generate the following
   Tcl functions:

   new_SwiftBlob()    -> SwiftBlob*
   delete_SwiftBlob() -> void
   SwiftBlob_pointer_set/get
   SwiftBlob_length_set/get
*/
typedef struct
{
  void* pointer;
  int length;
} SwiftBlob;

/**
   @param pointer An int representation of a void*
 */
SwiftBlob* SwiftBlob_create(long pointer, int length);

/**
   Allocate memory (not a blob) of given size
 */
void* SwiftBlob_malloc(int bytes);

/**
   Deallocate a blob
 */
void SwiftBlob_free(SwiftBlob* blob);

/**
   Obtain sizeof(double) (In Swift/T, all floats are 64-bit)
 */
int SwiftBlob_sizeof_float(void);

/*
   Cast functions for simple SWIG type transformation
   Not all possible type pairs are yet implemented
   If not given, void* is assumed
 */
void* SwiftBlob_cast_to_ptr(int i);

int SwiftBlob_cast_to_int(void* p);

double* SwiftBlob_cast_int_to_dbl_ptr(int i);
double* SwiftBlob_cast_to_dbl_ptr(void* p);

/**
   Assume blob is array of double- do array lookup
 */
double SwiftBlob_double_get(double* pointer, int index);

/**
   Assume blob is array of double- do array store
 */
void SwiftBlob_store_double(void* p, int i, double d);

/**
   Assume blob is array of char- do array lookup
 */
char SwiftBlob_char_get(SwiftBlob* blob, int index);

SwiftBlob* SwiftBlob_make_test(void);
